#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <cstdlib>
#include <ctime> 
#include <list>
#include <math.h>

#include "random.h"
#include "unixtimer.h"

using namespace std;

#define F_MARKED 1
#define F_REMOVED 2

#define RAND_SOL 1
#define MIN_SOL 2

#define SAVE_NEW 1
#define SAVE_OVER 2

#define VERBOSE 0
#define EXP 0

// --- GLOBAL VARIABLES ---
int nr_cc = 0;
int nr_snp = 0; // total number of SNPs in sequence
int nr_gen = 0; // total number of genotypes
int nr_hap = 0;

int nr_free = 0; // free window size
int nr_locked = 0; // locked window size

double total_cov;
bool MODE_drop = false;
//MODE_count : 0 - count all, 1 - count founders only, 2 - count children only
int  MODE_count = 1;

int num_dropped = 0;
int num_merrors = 0;

// --- STRUCT DECLARATION --- 
struct Gen;
struct CComp{
	CComp(int i) {id = i; recomb = false;}
	int id, num_sol;
	int size;
	bool recomb;
	double cur_cov;
	
	vector<Gen*> member;

	Gen ** node;
	char ** min_cf1;
	char ** min_cf2;
};

struct Gen{
	Gen() {
		seq = new char[nr_snp];
		t[0] = new char[nr_snp];
		t[1] = new char[nr_snp];
		h[0] = new char[nr_snp];
		h[1] = new char[nr_snp];
		h_init[0] = new char[nr_free];
		h_init[1] = new char[nr_free];		
		flag = false;
		cc = 0; pid[0] = 0; pid[1] = 0; id = 0;
		h[0][0] = h[1][0] = h_init[0][0] = h_init[1][0] = '\0';
		old_saved = false;
		inherit[0] = inherit[1] = sav_inherit[0] = sav_inherit[1] = -1;
		inherit_set[0] = inherit_set[1] = false;
		orphan = false;
	}
	
	CComp * cc;
    int id;
	int pid[2];
	int h_count[2];
	
	int inherit[2];
	int sav_inherit[2];
	bool inherit_set[2];
    
	Gen * pp[2];
	vector<Gen*> child;
    char sex;
	char * seq; // genotype sequence
	
	char * t[2], * h[2], * p[2], * h_init[2];

    int flag; // flag for ordering connected components (F_MARKED/F_REMOVED)
	bool old_saved, orphan;
};
struct Hap{ int id, coverage; };

vector<CComp*> cc_graph; // connected components table containing all genotypes
Hap * hap_table;

// --- FUNCTION DECLARATION ---
void read_genotypes(string filename);
bool fill_template(Gen * g);

void rand_haps(Gen * g, int start);
void drop_inheritence(CComp * c);
void print_all();

double mylog(int cov);
int hap_string2int(char * hap , int start);
int hap_string2int(char * p , char * c , int start);
void hap_int2string(int hap_id , int size, char* ret_data);
int compute_free(int pos);
int compute_locked(int pos);

int add_hap(int id);
int del_hap(int id);
void remove_hap(int i);

double get_coverage();
int consistant(Gen * p, Gen * c, int pos , int p_num);
bool is_match(char * t, char * c, int size);
bool is_match(Gen * g, char * c, int set, int size);
char compliment(char c);

int traverse(CComp * c, int i, int seq_start, double cur_cov, double * min_cov, int mode);

CComp* get_cc(Gen * g, CComp * c);
void merge_cc(CComp * c1, CComp * c2);
void cc_swap(int cc, int a, int b);
void cc_save(CComp * c, int start, int mode);
bool cc_save_min(CComp * c, int i, int start, double new_cov, double * cur_cov);
void insert_sort_cc(CComp * c, CComp ** arr, int size);

// --- FUNCTIONS ---

void print_cc(CComp * c){
	for(int i=0;i<c->size;i++){
		fprintf(stderr,"%d : %s (%s) %s\n",c->node[i]->id,c->node[i]->h[0],c->node[i]->seq,c->node[i]->h[1]);
	}
}

void print_cc(CComp * c, int start){
	for(int i=0;i<c->size;i++){
		fprintf(stderr,"%d - ",c->node[i]->id);
		for(int ii=start;ii<start+nr_free;ii++){
			fprintf(stderr,"%c",c->node[i]->t[0][ii]);
		}
		fprintf(stderr," ");
		for(int ii=start;ii<start+nr_free;ii++){
			fprintf(stderr,"%c",c->node[i]->seq[ii]);
		}
		fprintf(stderr," ");
		for(int ii=start;ii<start+nr_free;ii++){
			fprintf(stderr,"%c",c->node[i]->t[1][ii]);
		}
		fprintf(stderr,"\n");
			
	}
}

void insert_sort_cc(CComp * c, CComp ** arr, int size){
	for(int i=0;i<size;i++){
		if(c->cur_cov < arr[i]->cur_cov){
			for(int ii=size;ii>i;ii--){
				arr[ii] = arr[ii-1];
			}
			arr[i] = c;
			return;
		}
	}
	arr[size] = c;
}

/*
    main function
*/
int main(int argc, char** argv)
{		
	string file;
	bool MODE_greedy = true; bool bad_param = false;
	int seed = 1;
	bool free_set = false; bool locked_set = false;

	for(int i=1;i<argc;i++){
		if( strncmp(argv[i], "-input", strlen("-input")) == 0 ) file = argv[++i];
		else if( strncmp(argv[i], "-free", strlen("-free")) == 0 ) { nr_free = atoi(argv[++i]); free_set = true; }
		else if( strncmp(argv[i], "-locked", strlen("-locked")) == 0 ) { nr_locked = atoi(argv[++i]); locked_set = true; }
		else if( strncmp(argv[i], "-seed", strlen("-seed")) == 0 ) seed = atoi(argv[++i]);
		else if( strncmp(argv[i], "-no_batching", strlen("-no_batching")) == 0 ) MODE_greedy = 0;
		else if( strncmp(argv[i], "-drop", strlen("-drop")) == 0 ) MODE_drop = atoi(argv[++i]);
		else if( strncmp(argv[i], "-count_all", strlen("-count_all")) == 0 ) MODE_count = 0;
		else bad_param = true;
	}
	
	if(bad_param || (file == "") ){
		fprintf(stderr,"Usage:\n\t%s [options] -input <input file>", argv[0] );
		fprintf(stderr,"\nOPTIONS:\n" );
		fprintf(stderr,"\t-free N      : free window size (default: automaticaly selected)\n" );
		fprintf(stderr,"\t-locked N    : locked window size (default: automaticaly selected)\n" );
		fprintf(stderr,"\t-seed N      : random generator seed (default: 1)\n" );
		fprintf(stderr,"\t-no_batching : turn off batching (default: use batching)\n" );
		fprintf(stderr,"\t-count_all   : compute entropy over all haplotypes (default: count founder haplotypes only)\n" );
		return 0;
	} else if(file == ""){
		fprintf(stderr,"usage: %s -input <input file> -free <free window size (variable)> -locked <locked window size (variable)> -seed <random number seed (1)> -greedy <turn greedy mode on/off 1 or 0 (1) -count <0: all / 1: founders / 2: children>\n",argv[0]);
		fprintf(stderr,"error: no input file path provided\n");
		return 0;
	} else if(free_set && nr_free <= 0){
		fprintf(stderr,"error: free window size must be greater than zero\n");
		return 0;
	} else if(locked_set && nr_locked < 0){
		fprintf(stderr,"error: locked window size must be at least zero\n");
		return 0;
	}

#if VERBOSE >= 1
	char str_free[100], str_locked[100];
	if (free_set) sprintf(str_free,"%d",nr_free); else sprintf(str_free,"variable");
	if (locked_set) sprintf(str_locked,"%d",nr_locked); else sprintf(str_locked,"variable");
	fprintf(stderr,"processing %s - %s free, %s locked\n",file.c_str(),str_free,str_locked);
#endif

//	long start_time = time(NULL);
	init_rand(seed);
	read_genotypes(file.c_str());
	
	int min_cc;
	double min_cov, cur_cov;

	int sav_locked = nr_locked; int sav_free = nr_free;
	int cc_graph_size = cc_graph.size();

#if EXP
	int total_sol = 0;
	int num_iterations = 0;	
#endif
	int num_recomb = 0;
	int nr_win = 0; int tot_locked = 0; int tot_free = 0;

	CComp ** sorted_config;
	int sorted_size;
	bool recomb_clear = false, dropped = false;
	
	if(MODE_greedy) sorted_config = new CComp*[cc_graph_size];

	start_timer();
	
	// for each window
	for(int i=0 ; i<nr_snp ; i+=nr_free){
		if(nr_win % 200 == 199) fprintf(stderr,".");
		nr_win++;
		if(!free_set) { nr_free = sav_free = compute_free(i); tot_free += nr_free; }
		if(!locked_set) { nr_locked = sav_locked = compute_locked(i); tot_locked += nr_locked; }
			
		// if we used a small nr_free in the last window, return to the original
		if(nr_free != sav_free) nr_free = sav_free;
		if(nr_locked != sav_locked) nr_locked = sav_locked;

		if(i-nr_locked < 0) nr_locked = i;
		if(i+nr_free > nr_snp) sav_free = nr_free = nr_snp - i;

		// clear the hap list
		nr_hap = 0;

		// generate random haplotypes and populate hap_list
		for(int ii=0;ii<cc_graph.size();ii++){
#if VERBOSE >= 1
			fprintf(stderr,".");
#endif
			cc_graph[ii]->num_sol = 0;
			
			for(int iii=cc_graph[ii]->size-1;iii>=0;iii--) {
				cc_graph[ii]->node[iii]->h_count[0] = cc_graph[ii]->node[iii]->h_count[1] = 0;
			}
			
			if(cc_graph[ii]->recomb){
				// recombination occured - drop its inheritence pattern
				drop_inheritence(cc_graph[ii]);
				cc_graph[ii]->recomb = false;
			}
			
			traverse(cc_graph[ii],cc_graph[ii]->size-1,i,0,(double*)NULL,RAND_SOL);
#if EXP
			total_sol += cc_graph[ii]->num_sol;
#endif

			// --- check for recombinations
			recomb_clear = false;
			while(cc_graph[ii]->num_sol == 0) {
				if(recomb_clear){
					// we have dropped the inheritence pattern but still no results
					fprintf(stderr,"no solution - No:%d SNP:%d\n",cc_graph[ii]->node[0]->id,i);
					if(!MODE_drop) exit(0);
					fprintf(stderr,"dropping pedigree\n");
					// -- drop this pedigree
					nr_gen -= cc_graph[ii]->size;
					cc_graph.erase(cc_graph.begin() + ii);
					dropped = true;
					num_dropped++;
					nr_free = sav_free;
					break;
				}
					
				cc_graph[ii]->recomb = true; // mark this as having a recombinant event
				nr_free--;
					
				if(nr_free == 0){
					num_recomb++;
					// drop the inheritence pattern
					drop_inheritence(cc_graph[ii]);
					cc_graph[ii]->recomb = false;
					nr_free = 1;
					recomb_clear = true;
				}
				traverse(cc_graph[ii],cc_graph[ii]->size-1,i,0,(double*)NULL,RAND_SOL);
			}
			if(dropped) {
				dropped = false;
				ii--;
				continue;
			}
			if(cc_graph[ii]->recomb){num_recomb++;}
			// continue the remaining pedigree and ent_phasing with new nr_free value
			cc_save(cc_graph[ii],i,SAVE_NEW);
		}
#if VERBOSE >= 1
		fprintf(stderr,"\n");
#endif
		
		// compute & save total entropy
		total_cov = min_cov = get_coverage();

		double dif_ent = -1;
		while( dif_ent == -1 || dif_ent > 0 ){
			sorted_size = 0;
#if VERBOSE >= 1
			double total_entropy = (-1.0/(2.0*nr_gen))*(-2.0*nr_gen*log(2.0*nr_gen)/log(2.0) - total_cov);
			fprintf(stderr,"min entropy = %f window [%d - %d]\n",total_entropy,i-nr_locked,i+nr_free-1);
#endif
			
			// traverse each connected component
			for(int ii=0;ii<cc_graph_size;ii++){
				
				if(cc_graph[ii]->num_sol != 1){
					cc_graph[ii]->num_sol = 0;
					cur_cov = min_cov;
					
					for(int iii=0;iii<cc_graph[ii]->size;iii++) 
						cc_graph[ii]->node[iii]->h_count[0] = cc_graph[ii]->node[iii]->h_count[1] = 0;
					
					traverse(cc_graph[ii],cc_graph[ii]->size-1,i,total_cov,&cur_cov,MIN_SOL);
					
					// restore the original haplotypes for this node
					for(int iii=0;iii<cc_graph[ii]->size;iii++){
						strncpy(cc_graph[ii]->node[iii]->h[0]+i,cc_graph[ii]->node[iii]->h_init[0],nr_free);
						strncpy(cc_graph[ii]->node[iii]->h[1]+i,cc_graph[ii]->node[iii]->h_init[1],nr_free);
					}

#if VERBOSE >= 2
					fprintf(stderr,"CCNo.%d: num feasable solutions = %d\n",ii,cc_graph[ii]->num_sol);
#endif
#if EXP
					num_iterations++;
#endif
					if(MODE_greedy) {
						cc_graph[ii]->cur_cov = cur_cov;
						insert_sort_cc(cc_graph[ii],sorted_config,sorted_size++);
					}

					if(cur_cov < (min_cov - 1e-9)){
						min_cov = cur_cov;
						min_cc = ii;
					}
				}
			}
			
			// if we generated a smaller entropy
			if(min_cov < (total_cov - 1e-9)) {
				// GREEDY: go through all components and save if they're smaller
				if(MODE_greedy){
					cc_save(sorted_config[0],i,SAVE_OVER);
					for(int ii=1;ii<sorted_size;ii++) {
						if(cc_save_min(sorted_config[ii],0,i,min_cov,&min_cov)){
#if VERBOSE >= 1
//							total_entropy = (-1.0/(2.0*nr_gen))*(-2.0*nr_gen*log(2.0*nr_gen)/log(2.0) - min_cov);
//							fprintf(stderr," secondary minimal solution found - %f\n",total_entropy);
#endif
						}
					}
				// NORMAL: save only the minimal component
				} else { cc_save(cc_graph[min_cc],i,SAVE_OVER); }
			}
			dif_ent = total_cov - min_cov;
			total_cov = min_cov;
		}		
		// for each unsaved component, save it
#if VERBOSE >= 1
//		fprintf(stderr,"switching to next window (%ds)\n",time(NULL)-start_time);
#endif
	}

	print_all();
//if VERBOSE >= 1	

	fprintf(stderr,"\nTotal runtime: %.2f\n",cpu_seconds());

#if VERBOSE >= 1	
	if(!free_set) printf("avg free window size = %.1f\n",tot_free/(double)nr_win);
	if(!locked_set) printf("avg locked window size = %.1f\n",tot_locked/(double)nr_win);
	printf("\nrecombinations introduced: %d\n", num_recomb);
	printf("mendelian errors: %d\n", num_merrors);
	if(MODE_drop) printf("dropped pedigrees: %d\n", num_dropped);
#endif


	return 0;
}

void drop_inheritence(CComp * c){
	for(int i=0;i<c->size;i++) {
		c->node[i]->inherit[0] = c->node[i]->inherit[1] = -1;
		c->node[i]->inherit_set[0] = c->node[i]->inherit_set[1] = false;
	}
}

/*
	compute the free window size by looking at the number of ambiguous positions in all sequences
*/
int compute_free(int pos){
	int max = 2 * nr_gen;
	int nr_ambig = 0;
	int i = pos;
	
	while(nr_ambig <= max && i < nr_snp){
		for(int n=0;n<cc_graph.size();n++){
			for(int nn=0;nn<cc_graph[n]->size;nn++){
				if(cc_graph[n]->node[nn]->seq[i] == '2' || cc_graph[n]->node[nn]->seq[i] == '?'){
					nr_ambig++;
				}
			}
		}
		i++;
		if( (i-pos) > 5 ) return 5;
	}
	
	if( (i-pos) <= 1) return 1;
	else return (i-pos) - 1;
}

int compute_locked(int pos){
        int max = 2 * nr_gen;
        int nr_ambig = 0;
        int i = pos;

        while(nr_ambig <= max && i >= 0){
                for(int n=0;n<cc_graph.size();n++){
                        for(int nn=0;nn<cc_graph[n]->size;nn++){             
                                if(cc_graph[n]->node[nn]->seq[i] == '2' || cc_graph[n]->node[nn]->seq[i] == '?'){
                                        nr_ambig++;
                                }
                        }
                }
                i--;
                if( (pos - i) > 5 ) return 5;
        }

        if( (pos - i) <= 1) return 1;
        else return (pos - i) - 1;
}


/*
	print each connected component in order
*/
void print_all(){
	printf("%d %d\n",nr_gen,nr_snp);

	for(int i=0;i<cc_graph.size();i++){
		for(int ii=0;ii<cc_graph[i]->size;ii++){
			printf("%d %c %d %d\n",cc_graph[i]->node[ii]->id,cc_graph[i]->node[ii]->sex,cc_graph[i]->node[ii]->pid[0],cc_graph[i]->node[ii]->pid[1]);
			for(int iii=0;iii<nr_snp;iii++){
				printf("%c",cc_graph[i]->node[ii]->h[0][iii]);
			}
			printf("\n");
			for(int iii=0;iii<nr_snp;iii++){
				printf("%c",cc_graph[i]->node[ii]->h[1][iii]);
			}
			printf("\n");

		}
	}
}

void print(char * c,int start,int end){
	for(int i=start;i<end;i++){
		cout << c[i];
	}
}

double mylog(int cov){
	if(cov <=0 || nr_gen <=0) return 0;
	double p = (double)cov;
	return p*log(p)/log((double)2);
}

/* compute overall entropy from hap_list */
double get_coverage()
{
	double entropy = 0.0;

	for(int i=0;i<nr_hap;i++){
		double p = (double)hap_table[i].coverage;
		entropy += p*log(p);
	}

	entropy *= (-1);	
	entropy /= log((double)2);
	return entropy;
}

void cc_save(CComp * c, int start, int mode){
	// iterate through each node and set its haplotypes (and template)

	for(int i=0;i<c->size;i++){
		
		// remove the old haplotypes
		if( mode == SAVE_OVER && ( MODE_count == 0 || (MODE_count == 1 && c->node[i]->orphan) || (MODE_count == 2 && !c->node[i]->orphan)) ){
			del_hap(hap_string2int(c->node[i]->h[0],c->node[i]->h_init[0],start-nr_locked));
			del_hap(hap_string2int(c->node[i]->h[1],c->node[i]->h_init[1],start-nr_locked));
		}
		
		// copy the min config into the haplotypes
		strncpy(c->node[i]->h[0]+start,c->min_cf1[i],nr_free);
		strncpy(c->node[i]->h[1]+start,c->min_cf2[i],nr_free);
		// copy the min config into the initial position
		strncpy(c->node[i]->h_init[0],c->min_cf1[i],nr_free);
		strncpy(c->node[i]->h_init[1],c->min_cf2[i],nr_free);
		
		// save the inheritance values if need be
		if(c->node[i]->pid[0] != 0){
			if(!c->node[i]->inherit_set[0] && c->node[i]->sav_inherit[0] != -1) {
				c->node[i]->inherit[0] = c->node[i]->sav_inherit[0];
				c->node[i]->inherit_set[0] = true;
			} else if(start > 0){
				for(int j=start;j<start+nr_free;j++){
					if(c->node[i]->pp[0]->h[0][j] != c->node[i]->pp[0]->h[1][j]){
						if(c->node[i]->h[0][j] == c->node[i]->pp[0]->h[0][j]){
							c->node[i]->inherit[0] = 0;
							c->node[i]->inherit_set[0] = true; break;
						} else
						if(c->node[i]->h[0][j] == c->node[i]->pp[0]->h[1][j]){
							c->node[i]->inherit[0] = 1;
							c->node[i]->inherit_set[0] = true; break;
						} else { fprintf(stderr, "error: saving haplotypes: Id No.%d\n",c->node[i]->id); exit(0); }
					}
				}
			}
		}
		if(c->node[i]->pid[1] != 0){
			if(!c->node[i]->inherit_set[1] && c->node[i]->sav_inherit[1] != -1) {
				c->node[i]->inherit[1] = c->node[i]->sav_inherit[1];
				c->node[i]->inherit_set[1] = true;
			} else if(start > 0){
				for(int j=start;j<start+nr_free;j++){
					if(c->node[i]->pp[1]->h[0][j] != c->node[i]->pp[1]->h[1][j]){
						if(c->node[i]->h[1][j] == c->node[i]->pp[1]->h[0][j]){
							c->node[i]->inherit[1] = 0;
							c->node[i]->inherit_set[1] = true; break;
						} else
						if(c->node[i]->h[1][j] == c->node[i]->pp[1]->h[1][j]){
							c->node[i]->inherit[1] = 1;
							c->node[i]->inherit_set[1] = true; break;
						} else { fprintf(stderr, "error: saving haplotypes: Id No.%d\n",c->node[i]->id); exit(0); }
					}
				}
			}
		}
		
		// add the new ones in hap_list
		if( MODE_count == 0 || (MODE_count == 1 && c->node[i]->orphan) || (MODE_count == 2 && !c->node[i]->orphan) ){
			add_hap(hap_string2int(c->node[i]->h[0],start-nr_locked));
			add_hap(hap_string2int(c->node[i]->h[1],start-nr_locked));
		}
		
	}
}

/*
	save the configuration if its entropy is smaller than the given
*/
bool cc_save_min(CComp * c, int i, int start, double new_cov, double * cur_cov){

	if(i==c->size){
		if(new_cov < (*cur_cov - 1e-9)) {
			*cur_cov = new_cov;
			return true;
		} else return false;
	}

	bool save = true;

	strncpy(c->node[i]->h[0]+start,c->min_cf1[i],nr_free); strncpy(c->node[i]->h[1]+start,c->min_cf2[i],nr_free);

	// --- remove the old haplotypes and add the new ones in hap_list
	int new1_id = hap_string2int(c->node[i]->h[0],start-nr_locked);
	int new2_id = hap_string2int(c->node[i]->h[1],start-nr_locked);
	int old1_id = hap_string2int(c->node[i]->h[0],c->node[i]->h_init[0],start-nr_locked);
	int old2_id = hap_string2int(c->node[i]->h[1],c->node[i]->h_init[1],start-nr_locked);
	
	int old1_cov, old2_cov, new1_cov, new2_cov;

	old1_cov = del_hap(old1_id);
	new_cov = new_cov + mylog(old1_cov+1) - mylog(old1_cov);
	old2_cov = del_hap(old2_id);
	new_cov = new_cov + mylog(old2_cov+1) - mylog(old2_cov);

	new1_cov = add_hap(new1_id);
	new_cov = new_cov + mylog(new1_cov-1) - mylog(new1_cov);
	new2_cov = add_hap(new2_id);
	new_cov = new_cov + mylog(new2_cov-1) - mylog(new2_cov);

	if(!cc_save_min(c,i+1,start,new_cov,cur_cov)){
		// restore the old haps
		
		strncpy(c->node[i]->h[0]+start,c->node[i]->h_init[0],nr_free);
		strncpy(c->node[i]->h[1]+start,c->node[i]->h_init[1],nr_free);
		
		add_hap(old1_id);
		add_hap(old2_id);
		del_hap(new1_id);
		del_hap(new2_id);
		save = false;
	} else {

		strncpy(c->node[i]->h_init[0],c->min_cf1[i],nr_free); strncpy(c->node[i]->h_init[1],c->min_cf2[i],nr_free);
		
		if(c->node[i]->pid[0] != 0){
			if(!c->node[i]->inherit_set[0] && c->node[i]->sav_inherit[0] != -1) {
				c->node[i]->inherit[0] = c->node[i]->sav_inherit[0];
				c->node[i]->inherit_set[0] = true;
			} else if(start > 0){
				for(int j=start;j<start+nr_free;j++){
					if(c->node[i]->pp[0]->h[0][j] != c->node[i]->pp[0]->h[1][j]){
						if(c->node[i]->h[0][j] == c->node[i]->pp[0]->h[0][j]){
							c->node[i]->inherit[0] = 0;
							c->node[i]->inherit_set[0] = true; break;
						} else
						if(c->node[i]->h[0][j] == c->node[i]->pp[0]->h[1][j]){
							c->node[i]->inherit[0] = 1;
							c->node[i]->inherit_set[0] = true; break;
						} else { fprintf(stderr, "error: saving haplotypes: Id No.%d\n",c->node[i]->id); exit(0); }
					}
				}
			}
		}
		
		if(c->node[i]->pid[1] != 0){
			if(!c->node[i]->inherit_set[1] && c->node[i]->sav_inherit[1] != -1) {
				c->node[i]->inherit[1] = c->node[i]->sav_inherit[1];
				c->node[i]->inherit_set[1] = true;
			} else if(start > 0){
				for(int j=start;j<start+nr_free;j++){
					if(c->node[i]->pp[1]->h[0][j] != c->node[i]->pp[1]->h[1][j]){
						if(c->node[i]->h[1][j] == c->node[i]->pp[1]->h[0][j]){
							c->node[i]->inherit[1] = 0;
							c->node[i]->inherit_set[1] = true; break;
						} else
						if(c->node[i]->h[1][j] == c->node[i]->pp[1]->h[1][j]){
							c->node[i]->inherit[1] = 1;
							c->node[i]->inherit_set[1] = true; break;
						} else { fprintf(stderr, "error: saving haplotypes: Id No.%d\n",c->node[i]->id); exit(0); }
					}
				}
			}
		}
	}
	
	return save;
}

/*
	hap <-> int hash functions (converts '0'/'1' sequence into bitstring)
*/
void hap_int2string(int hap_id , int size, char * ret_data)
{
	int rest, i;
	ret_data[size]='\0';
	i = size - 1;
	rest = hap_id;
	
	while ( i >= 0 )
	{
		if(rest % 2 == 1) 
			*(ret_data + i) ='1';
		else
			*(ret_data + i) ='0';
		rest /= 2;
		i-- ;
	}

}

/* generate int for haplotype seq front start to window size */
int hap_string2int(char * hap , int start)
{
	int ret_data = hap[start] - '0';
	for(int i=start+1;i<start+nr_free+nr_locked;i++){
		ret_data <<=1;
		ret_data += hap[i] - '0';
	}

	return ret_data;
}

/* generate int for haplotype using last (locked) portion of p and first (free) portion of c */
int hap_string2int(char * p , char * c , int start)
{
	int ret_data;

	if(nr_locked > 0) {
		ret_data = p[start] - '0';
		for(int i=start+1;i<start+nr_locked;i++){
			ret_data <<= 1;
			ret_data += p[i] - '0';
		}
		for(int i=0;i<nr_free;i++){
			ret_data <<= 1;
			ret_data += c[i] - '0';
		}
	} else {
		ret_data = c[0] - '0';
		for(int i=1;i<nr_free;i++){
			ret_data <<= 1;
			ret_data += c[i] - '0';
		}
	}

	return ret_data;
}

bool is_match(char * t, char * c, int size){
	for(int i=size-nr_free;i<size;i++){
		if(t[i] == '?') continue;
		else if(t[i] != c[i]) return false;
	}
	return true;
}

bool is_match(Gen * g, char * c, int set, int size){
	for(int i=size-nr_free;i<size;i++){
		if(g->t[1-set][i] == '?') {
			if(g->seq[i] == '2'){
				if(g->h[set][i] != compliment(c[i])) return false;
			}
			continue;
		}
		else if(g->t[1-set][i] != c[i]) return false;
	}
	return true;
}	

int consistant(Gen * p, Gen * c, int pos , int p_num){
	
	/*
		return 0 - the child did not match either of the fathers haplotypes
		return 1 - the child matches one of the parents haplotypes
		return 2 - neither of the parents haplotypes has been set - and the child matches both (ambiguous - try both)
		return 3 - one haplotype is set in the parent, child matches it and the unset haplotype (ambiguous - do not try both, do not save)
	*/
	
	if(!c->inherit_set[p_num]){
		// neither haplotype has been used
		if(p->h_count[0] == 0 && p->h_count[1] == 0){ // both parent haplotypes are unused
			if(is_match(p->t[0],c->h[p_num],pos)){
				if(is_match(p->t[1],c->h[p_num],pos)) {
					return 2; // both parent haplotypes match
				} else {
					c->inherit[p_num] = 0;
				}
			} else if(is_match(p->t[1],c->h[p_num],pos)){
				c->inherit[p_num] = 1;
			} else {
				return false;
			}
		} else if(p->h_count[0] == 0){
			// p_h0 is free, see if we match p_h1
			if( strncmp(p->h[1]+pos-nr_free,c->h[p_num]+pos-nr_free,nr_free) == 0 ){
				// h1 is transmitted
				c->inherit[p_num] = 1;
				if(is_match(p,c->h[p_num],1,pos)){ return 3; }
				
			} else if(is_match(p,c->h[p_num],1,pos)){
				// h0 is set & transmitted
				c->inherit[p_num] = 0;
			} else {
				return false;
			}
		} else if(p->h_count[1] == 0){
			// p_h1 is free, see if we match p_h0
			if( strncmp(p->h[0]+pos-nr_free,c->h[p_num]+pos-nr_free,nr_free) == 0 ){
				// h0 is transmitted
				c->inherit[p_num] = 0;
				if(is_match(p,c->h[p_num],0,pos)){ return 3; }
				
			} else if(is_match(p,c->h[p_num],0,pos)){
				// h1 is set & transmitted
				c->inherit[p_num] = 1;
			} else {
				return false;
			}
		} else {
			// both have been set
			if( strncmp(p->h[0]+pos-nr_free,c->h[p_num]+pos-nr_free,nr_free) == 0 ){
				// h1 is transmitted
				c->inherit[p_num] = 0;
			} else
			if( strncmp(p->h[1]+pos-nr_free,c->h[p_num]+pos-nr_free,nr_free) == 0 ){
				// h2 is transmitted
				c->inherit[p_num] = 1;
			} else {
				// neither matches
				return false;
			}
		}
	} else { // the transmitted haplotype has been detirmined

		int inherit = c->inherit[p_num];
		if(p->h_count[inherit] == 0){
			if( !is_match(p->t[inherit]+pos-nr_free,c->h[p_num]+pos-nr_free,nr_free) ){
				return false;
			}
		} else if( strncmp(p->h[inherit]+pos-nr_free,c->h[p_num]+pos-nr_free,nr_free) != 0 ){
			return false;
		}
	}
	return 1;
}

/*
	traverse the connected component and find haps that create minimal overall entropy
*/
int traverse(CComp * c, int i, int seq_start, double cur_cov, double * min_cov, int mode){
	
	// we are at a leaf
	if(i == -1){
		c->num_sol++;
		if(mode == MIN_SOL){
			// if the coverage is smaller (or no solution has been found), return it
			if(cur_cov < (*min_cov - 1e-9)) {
				*min_cov = cur_cov;
				return 1;
			} else return 2;

		} else if(mode == RAND_SOL){
			// if this is the first solution - save it
			if(c->num_sol == 1){
				return 1;
			// otherwise, randomly save it
			} else if(unif_rand(c->num_sol) == 0){
				return 1;
			} else {
				return 2;
			}
		}
		fprintf(stderr,"error: no mode selected for traversal\n");
		exit(0);
	}
	
	int save = 0; int cur_return;
	bool error = false;
	bool tmp_inherit_set[2]; tmp_inherit_set[0] = tmp_inherit_set[1] = false;

	bool h_set[2]; h_set[0] = c->node[i]->h_count[0] > 0; h_set[1] = c->node[i]->h_count[1] > 0;
	bool h_set_tmp[2]; h_set_tmp[0] = h_set_tmp[1] = false;
	bool h_neither = !(h_set[0] || h_set[1]);
	
	int old1_id, old2_id, h1_id, h2_id, h1_cov, h2_cov, old1_cov, old2_cov;
	double sav_cov;
	
	if(mode == MIN_SOL && ( MODE_count == 0 || (MODE_count == 1 && c->node[i]->orphan) || (MODE_count == 2 && !c->node[i]->orphan)) ){
		old1_id = hap_string2int(c->node[i]->h[0],c->node[i]->h_init[0],seq_start-nr_locked);
		old2_id = hap_string2int(c->node[i]->h[1],c->node[i]->h_init[1],seq_start-nr_locked);
	}
	
	// clear out the haplotypes so they can be re-detirmined
	for(int ii=seq_start;ii<seq_start+nr_free;ii++){
		if(!h_set[0]) c->node[i]->h[0][ii] = '\0';
		if(!h_set[1]) c->node[i]->h[1][ii] = '\0';
	}
	
	int consis_0, consis_1;
	
	int k;
	
	// if(h_set[0] && h_set[1]) k = seq_start + nr_free;
	k = seq_start;
	
	char cur_cmp;
	
	while(k>=seq_start){
		// start: has the entire sequence been filled?
		if(k==seq_start + nr_free){		
			consis_0 = consis_1 = -1;
			
			// set the transmitted haplotypes
			if(c->node[i]->pid[0] != 0){
				consis_0 = consistant(c->node[i]->pp[0],c->node[i],k,0);
			}
			if(c->node[i]->pid[1] != 0){
				consis_1 = consistant(c->node[i]->pp[1],c->node[i],k,1);
			}

			if ( consis_0 == 0 || consis_1 == 0 )
			{
					if ( h_set[0] && h_set[1] ) return false;
					else { k--; continue; }
			}
			
			if(mode == MIN_SOL && ( MODE_count == 0 || (MODE_count == 1 && c->node[i]->orphan) || (MODE_count == 2 && !c->node[i]->orphan)) ){
				// -- increment these haps into the hap_table
				// save all hap ids

				sav_cov = cur_cov;

				h1_id = hap_string2int(c->node[i]->h[0],seq_start-nr_locked);
				h2_id = hap_string2int(c->node[i]->h[1],seq_start-nr_locked);
				// remove the original haps

				old1_cov = del_hap(old1_id);
				cur_cov = cur_cov + mylog(old1_cov+1) - mylog(old1_cov);
				old2_cov = del_hap(old2_id);
				cur_cov = cur_cov + mylog(old2_cov+1) - mylog(old2_cov);

				// add the new haps
				h1_cov = add_hap(h1_id);
				cur_cov = cur_cov + mylog(h1_cov-1) - mylog(h1_cov);
				h2_cov = add_hap(h2_id);
				cur_cov = cur_cov + mylog(h2_cov-1) - mylog(h2_cov);
			}

			bool parent_done = false;
			int trav_ctr = 0;
			
			while(!parent_done){
				
				if(consis_0 == 2 && consis_1 != 1){ // try both positions in both parents
					switch (trav_ctr) {
						case 0:
							c->node[i]->inherit[0] = 0;
							c->node[i]->inherit[1] = 0;
							break;
						case 1:
							c->node[i]->inherit[0] = 0;
							c->node[i]->inherit[1] = 1;
							break;	
						case 2:
							c->node[i]->inherit[0] = 1;
							c->node[i]->inherit[1] = 0;
							break;
						case 3:

							c->node[i]->inherit[0] = 1;
							c->node[i]->inherit[1] = 1;
							parent_done = true;
							break;
					}
				} else if(consis_0 == 2){ // try both positions in first parent
					switch (trav_ctr) {
						case 0:
							c->node[i]->inherit[0] = 0;
							break;
						case 1:
							c->node[i]->inherit[0] = 1;
							parent_done = true;
							break;
					}
				} else if(consis_1 == 2){ // try both positions in second parent
					switch (trav_ctr) {
						case 0:
							c->node[i]->inherit[1] = 0;
							break;
						case 1:
							c->node[i]->inherit[1] = 1;
							parent_done = true;
							break;
					}
				} else { // only try the preset positions

					parent_done = true;
				}
				
				// copy the haplotypes into the parent
				if(consis_0 != -1) {
					if(++c->node[i]->pp[0]->h_count[c->node[i]->inherit[0]] == 1) {
						strncpy(c->node[i]->pp[0]->h[c->node[i]->inherit[0]]+seq_start,c->node[i]->h[0]+seq_start,nr_free);
					}
				}
				if(consis_1 != -1) {
					if(++c->node[i]->pp[1]->h_count[c->node[i]->inherit[1]] == 1) {
						strncpy(c->node[i]->pp[1]->h[c->node[i]->inherit[1]]+seq_start,c->node[i]->h[1]+seq_start,nr_free);
					}
				}
			
				// go to the next node
				cur_return = traverse(c,i-1,seq_start,cur_cov,min_cov,mode);
				if( cur_return != 0 ){
					parent_done = true; // we have found a solution, stop swapping
					if(cur_return == 1) {
						// an entropy is returned - save current configuration
						strncpy(c->min_cf1[i],c->node[i]->h[0]+seq_start,nr_free);
						strncpy(c->min_cf2[i],c->node[i]->h[1]+seq_start,nr_free);

						if(c->node[i]->pid[0] != 0 && consis_0 == 1)
							c->node[i]->sav_inherit[0] = c->node[i]->inherit[0];
						else c->node[i]->sav_inherit[0] = -1;
						if(c->node[i]->pid[1] != 0 && consis_1 == 1)
							c->node[i]->sav_inherit[1] = c->node[i]->inherit[1];
						else c->node[i]->sav_inherit[1] = -1;
						save = 1;
					} else if(save == 0) { save = 2; }
				}
				
				// lower the counts of the parent
				if(consis_0 != -1) c->node[i]->pp[0]->h_count[c->node[i]->inherit[0]]--;
				if(consis_1 != -1) c->node[i]->pp[1]->h_count[c->node[i]->inherit[1]]--;
				
				trav_ctr++;
			}

			if(mode == MIN_SOL && ( MODE_count == 0 || (MODE_count == 1 && c->node[i]->orphan) || (MODE_count == 2 && !c->node[i]->orphan)) ){
				// --- restore the old haps into the hap table
				// add the old haps
				add_hap(old1_id);
				add_hap(old2_id);
				// remove the new haps
				del_hap(h1_id);
				del_hap(h2_id);
				// restore cur_cov
				cur_cov = sav_cov;
			}
			
			if(h_set[0] && h_set[1]) return save; // both haplotypes are pre-set, we are done here
			else k--; // otherwise, continue to the next hap variant
		} else {
			// start: found an ambiguity - fill it
			
			// if both haplotypes are set, make sure they are consistent with the sequence
			if(h_set[0] && h_set[1]) {
				if(c->node[i]->seq[k] == '2'){
					if(c->node[i]->h[0][k] != compliment(c->node[i]->h[1][k])) return false;
				} 
				k++; continue;
			}
			
			if(c->node[i]->t[0][k] == '?' || c->node[i]->t[1][k] == '?'){
				if(c->node[i]->t[0][k] != '?' && !h_set[0]){
					if(h_set[1]){
						if(c->node[i]->h[0][k] == c->node[i]->t[0][k]) {
							c->node[i]->h[0][k] = '\0';
							k--; continue;
						} else {
							c->node[i]->h[0][k] = c->node[i]->t[0][k];
							k++; continue;
						}
					} else { c->node[i]->h[0][k] = c->node[i]->t[0][k]; }
					
					// otherwise temporarily set this one
					h_set[0] = true; h_neither = false;
					h_set_tmp[0] = true; 
				}
				if(c->node[i]->t[1][k] != '?' && !h_set[1]){
					if(h_set[0]){
						if(c->node[i]->h[1][k] == c->node[i]->t[1][k]) {
							c->node[i]->h[1][k] = '\0';
							k--; continue;
						} else {
							c->node[i]->h[1][k] = c->node[i]->t[1][k];
							k++; continue;
						}
					} else { c->node[i]->h[1][k] = c->node[i]->t[1][k]; }
					
					// otherwise temporarily set this one
					h_set[1] = true; h_neither = false;
					h_set_tmp[1] = true;
				}
					
					
				// start: iterate through each char combination
				if(c->node[i]->seq[k] == '2'){
					if(h_set[0]){ // one hap has been set, generate the compliment for it
						cur_cmp = compliment(c->node[i]->h[0][k]);
						if(c->node[i]->h[1][k] == cur_cmp){
							c->node[i]->h[1][k] = '\0';
							k--; continue;
						} else { c->node[i]->h[1][k] = cur_cmp; k++; }					
					} else 
					if(h_set[1]){
						cur_cmp = compliment(c->node[i]->h[1][k]);
						if(c->node[i]->h[0][k] == cur_cmp){
							c->node[i]->h[0][k] = '\0';
							k--; continue;
						} else { c->node[i]->h[0][k] = cur_cmp; k++; }
					} else { // neither hap has been set , generate both haplotypes		
						if(c->node[i]->h[0][k] == '0' && c->node[i]->h[1][k] == '1'){
							c->node[i]->h[0][k] = '1';
							c->node[i]->h[1][k] = '0';
							k++;
						}else
						if(c->node[i]->h[0][k] == '1' && c->node[i]->h[1][k] == '0'){
							c->node[i]->h[0][k] = '\0';
							c->node[i]->h[1][k] = '\0';
							k--; continue;
						}else
						{
							c->node[i]->h[0][k] = '0';
							c->node[i]->h[1][k] = '1';
							k++;
						}
					}
				} else
				if(c->node[i]->seq[k] == '?'){		
					if( c->node[i]->h[0][k] == '0' && c->node[i]->h[1][k] == '0' ){
						if(h_set[0] || h_neither){
							c->node[i]->h[0][k] = '0';
							c->node[i]->h[1][k] = '1';
						} else { // h1 is set
							c->node[i]->h[0][k] = '1';
						}
						k++;
					}else
					if( c->node[i]->h[0][k] == '0' && c->node[i]->h[1][k] == '1'){
						if(h_neither){
							c->node[i]->h[0][k] = '1';
							c->node[i]->h[1][k] = '0';
							k++;
						} else if(h_set[1]) {
							c->node[i]->h[0][k] = '1';
							k++;
						} else { // h0 is set
							c->node[i]->h[1][k] = '\0';
							k--; continue;
						}
					}else
					if(c->node[i]->h[0][k] == '1' && c->node[i]->h[1][k] == '0'){
						if(h_set[0] || h_neither){							
							c->node[i]->h[0][k] = '1';
							c->node[i]->h[1][k] = '1';
							k++;
						} else { // h1 is set
							c->node[i]->h[0][k] = '\0';
							k--; continue;
						}
					}else
					if(c->node[i]->h[0][k] == '1' && c->node[i]->h[1][k] == '1'){
						if(!h_set[0]) c->node[i]->h[0][k] = '\0';
						if(!h_set[1]) c->node[i]->h[1][k] = '\0';
						k--; continue;
					}else
					{
						if(!h_set[0]) c->node[i]->h[0][k] = '0';
						if(!h_set[1]) c->node[i]->h[1][k] = '0';
						k++;
					}
				} // end: iterate through each char combination
				
				if(h_set_tmp[0]){ h_set[0] = false; h_set_tmp[0] = false; h_neither = true;}
				if(h_set_tmp[1]){ h_set[1] = false; h_set_tmp[1] = false; h_neither = true;}
				
			} else if(c->node[i]->t[0][k] == c->node[i]->h[0][k] && c->node[i]->t[1][k] == c->node[i]->h[1][k]){
				if(!h_set[0]) c->node[i]->h[0][k] = '\0';
				if(!h_set[1]) c->node[i]->h[1][k] = '\0';
				k--; continue;
			} else {
				if(!h_set[0]) { c->node[i]->h[0][k] = c->node[i]->t[0][k];}
				if(!h_set[1]) { c->node[i]->h[1][k] = c->node[i]->t[1][k];}
				k++;
			} // end: found an ambiguity - fill it

			// does this match the template of parent's haplotype it comes from?
			if(c->node[i]->inherit_set[0]){
				if( c->node[i]->pp[0]->t[c->node[i]->inherit[0]][k-1] != '?' && 
					c->node[i]->pp[0]->t[c->node[i]->inherit[0]][k-1] != c->node[i]->h[0][k-1] ) { k--; continue; }
			}
			if(c->node[i]->inherit_set[1]){
				if( c->node[i]->pp[1]->t[c->node[i]->inherit[1]][k-1] != '?' && 
					c->node[i]->pp[1]->t[c->node[i]->inherit[1]][k-1] != c->node[i]->h[1][k-1] ) { k--; continue; }
			}
				
		} // end: has the entire sequence been filled?
	} // end: while

	return save;
}

char compliment(char c){
    if(c == '0') return '1';
    if(c == '1') return '0';
    return '?';
}

/*
    use info from parents to replace as many '?' as possible
*/
bool fill_template(Gen * g){
    /* assume t[0] comes from p[0] and t[1] comes from p[1] always */
	bool compSet;
	bool error;

	for(int i=0;i<nr_snp;i++){
		error = false;
        if(g->seq[i] == '2' || g->seq[i] == '?'){
			
			compSet = false;
			g->t[0][i] = g->t[1][i] = '?';
            if(g->pid[0] != 0){
                if(g->pp[0]->seq[i] != '2' && g->pp[0]->seq[i] != '?'){
					
					// set the t[0] from the parent
                    g->t[0][i] = g->pp[0]->seq[i];
					// if the SNP is not unknown, set its compliment in t[1]
					if(g->seq[i] == '2'){
						g->t[1][i] = compliment(g->t[0][i]);
						compSet = true;
					}
                }
			}
            if(g->pid[1] != 0){
                if(g->pp[1]->seq[i] != '2' && g->pp[1]->seq[i] != '?'){
					/* if a SNP was recovered from parent1 which does
						not match its compliment from parent2, error out
					*/
					if(compSet){
						if(g->t[1][i] != g->pp[1]->seq[i]) {
							fprintf(stderr,"1error: Mendelian Inconsistency\n");
							fprintf(stderr,"SNP(%d) - P1:%c P2:%c C:%c\n",i,g->pp[0]->seq[i],g->pp[1]->seq[i],g->seq[i]);
							g->pp[1]->seq[i] = g->pp[1]->t[0][i] = g->pp[1]->t[1][i] = '?';
							g->pp[0]->seq[i] = g->pp[0]->t[0][i] = g->pp[0]->t[1][i] = '?';
							g->seq[i] = g->t[0][i] = g->t[1][i] = '?';
							num_merrors++;
						}
					} else {
						g->t[1][i] = g->pp[1]->seq[i];
						if(g->seq[i] == '2') g->t[0][i] = compliment(g->t[1][i]);
					}
                }
			}
        } else if(g->seq[i] == '0' || g->seq[i] == '1'){ 
			if(g->pid[0] != 0){
				if(g->pp[0]->seq[i] != '2' && g->pp[0]->seq[i] != '?' && g->pp[0]->seq[i] != g->seq[i]){
					fprintf(stderr,"2error: Mendelian Inconsistency\n");
					//fprintf(stderr,"SNP(%d) - P1:%c P2:%c C:%c\n",i,g->pp[0]->seq[i],g->pp[1]->seq[i],g->seq[i]);
					g->pp[0]->seq[i] = g->pp[0]->t[0][i] = g->pp[0]->t[1][i] = '?';
					g->seq[i] = '?';
					error = true;
					num_merrors++;
				}
			}
			if(g->pid[1] != 0){
				if(error){ g->pp[1]->seq[i] = g->pp[1]->t[0][i] = g->pp[1]->t[1][i] = '?'; }
				else if(g->pp[1]->seq[i] != '2' && g->pp[1]->seq[i] != '?' && g->pp[1]->seq[i] != g->seq[i]) {
					fprintf(stderr,"3error: Mendelian Inconsistency\n");
					//fprintf(stderr,"SNP(%d) - P1:%c P2:%c C:%c\n",i,g->pp[0]->seq[i],g->pp[1]->seq[i],g->seq[i]);
					g->pp[1]->seq[i] = g->pp[1]->t[0][i] = g->pp[1]->t[1][i] = '?';
					if(g->pid[0] != 0) g->pp[0]->seq[i] = g->pp[0]->t[0][i] = g->pp[0]->t[1][i] = '?';
					g->seq[i] = '?';
					num_merrors++;
				}
			}
			g->t[0][i] = g->t[1][i] = g->seq[i];
		} else {
			fprintf(stderr,"sequence errror: must be of 0/1/2/? format\n");
			exit(0);
		}
    }
	return true;
}

/* 
	combine two connected components into one
*/
void merge_cc(CComp * c1, CComp * c2){
	for(int i=0;i<c1->member.size();i++){
		c1->member[i]->cc = c2;
		c2->member.push_back(c1->member[i]);
	}
	// remove the 1st CC and renumber the rest
	cc_graph.erase(cc_graph.begin()+c1->id);
	for(int i=0;i<cc_graph.size();i++){
		cc_graph[i]->id = i;
	}
	delete c1;
}

/*
	start with a node and build connected component for that node
*/
CComp* get_cc(Gen * g, CComp * c){

	// the current node is not in a component - add it to c
	if(g->cc == 0){
		g->cc = c;
		c->member.push_back(g);
		// traverse its children
		for(int i=0;i < g->child.size();i++){
			c = get_cc(g->child[i],c);
		}
	// this node is in a different component  - merge the two components
	} else if(g->cc->id != c->id) {
		merge_cc(c,g->cc);
		return g->cc;
	}
	// otherwise, the node is already in this component - do nothing
	return c;
}

/*
	swap two elements within a connected component
*/
void cc_swap(int cc, int a, int b){
	if(a==b) return;
	Gen * x = cc_graph[cc]->member[a];
	cc_graph[cc]->member[a] = cc_graph[cc]->member[b];
	cc_graph[cc]->member[b] = x;
}


/*
	use id to add to the coverage of hap in table
	or create the new hap
	return NEW coverage
*/
int add_hap(int id){

	for(int i=0;i<nr_hap;i++){
		if(hap_table[i].id == id){
			hap_table[i].coverage++;
			return hap_table[i].coverage;
		}
	}

	hap_table[nr_hap].id = id;
	hap_table[nr_hap].coverage = 1;

	nr_hap++;
	return 1;
}

/*
	remove a haplotype based on its ID
	returns the NEW coverage
*/

int del_hap(int id){

	// see if hap is in the table
	for(int i=0;i<nr_hap;i++){
		if(hap_table[i].id == id){
			if(hap_table[i].coverage == 1){
				remove_hap(i);
			} else {
				hap_table[i].coverage--;
				return hap_table[i].coverage;
			}
		}
	}
	return 0;
}

/* resize the array when deleting a haplotype */
void remove_hap(int i){
	for(int ctr=i;ctr<nr_hap;ctr++){
		hap_table[ctr] = hap_table[ctr+1];
	}
	nr_hap--;
}

Gen * get_node(Gen * orphan, Gen * gen_array, int pid){
//BGD
//	cout << "nr_gen "<<nr_gen<<" pid "<<pid<<endl;
	for(int i=1;i<=nr_gen;i++){
	//	cout << i <<" "<<gen_array[i].id<<endl;
		if(gen_array[i].id == pid){
			return &gen_array[i];
		}
	}
//	cout << "pid not found"<<endl;
	// parent was not found
	Gen * p = new Gen();
	p->id = pid; p->pp[0] = p->pp[1] = 0;
	for(int i=0;i<nr_snp;i++) p->seq[i] = '?';
	orphan->child.push_back(p);
	nr_gen++;
	return p;
}

/*
    parse file and create topological connected components for all gens
*/
void read_genotypes(string filename){
    /*
    file structure:
    <number of genomes> <number of snps>
    <genome id> <sex> <parent1 ID> <parent2 ID> <genome sequence>
    "
    */

	int order_ctr = 0, i=1;

    ifstream fin;
    fin.open(filename.c_str());
	if(!fin){
		fprintf(stderr,"cannot open file for input\n");
		exit(0);
	}
    fin >> nr_gen >> nr_snp;
#if VERBOSE >= 1
	fprintf(stderr,"# of genotypes = %d\n", nr_gen);
#endif
	Gen * gen_array = new Gen[nr_gen+1];
	hap_table = new Hap[2*nr_gen];
	gen_array[0].id = 0;

	// --- REGULAR INPUT ---
	// read each genotype from file
	for(int i=1;i<=nr_gen;i++){
		if(fin.eof()){
			fprintf(stderr,"input format incorrect: not enough sequences\n");
			exit(0);
		}
		fin >> gen_array[i].id >> gen_array[i].sex >> gen_array[i].pid[0] >> gen_array[i].pid[1] >> gen_array[i].seq;

		if(!gen_array[i].id) { fprintf(stderr,"input format incorrect: genotype ID(%d)\n",i); exit(0); 
}
		else if(!gen_array[i].sex) { fprintf(stderr,"input format incorrect: genotype sex\n"); exit(0); }
		else if(!gen_array[i].seq || strlen(gen_array[i].seq) != nr_snp){ fprintf(stderr,"input format incorrect: genotype sequence\n"); exit(0);}
	}		
	fin.close();

	// set all parent/child pointers
	bool orphan;
	Gen * temp_parent;
	int cur_nr_gen = nr_gen;
	for(int i=1;i<=cur_nr_gen;i++){
		orphan = true;
		if(gen_array[i].pid[0] != 0){
			orphan = false;
			temp_parent = get_node(&gen_array[0],gen_array,gen_array[i].pid[0]);
			gen_array[i].pp[0] = temp_parent;
			temp_parent->child.push_back(&gen_array[i]);
		} else {
			gen_array[i].pp[0] = 0;
		}

		if(gen_array[i].pid[1] != 0){
			orphan = false;
			temp_parent = get_node(&gen_array[0],gen_array,gen_array[i].pid[1]);
			gen_array[i].pp[1] = temp_parent;
			temp_parent->child.push_back(&gen_array[i]);
		} else {
			gen_array[i].pp[1]= 0;
		}

		if(orphan) {
			gen_array[i].orphan = true;
			gen_array[0].child.push_back(&gen_array[i]);
		}
	}

	// --- isolate connected components
	// traverse all nodes with no parents and build their connected components
	for(int i=0;i<gen_array[0].child.size();i++){
		CComp * new_cc = new CComp(cc_graph.size());
		cc_graph.push_back(new_cc);
		get_cc(gen_array[0].child[i],new_cc);
	}

	// order each connected component
	int ii=0;
	for(int i=0;i<cc_graph.size();i++){
		// create the min configuration array to qual the number of members in the component
		cc_graph[i]->min_cf1 = new char*[cc_graph[i]->member.size()];
		cc_graph[i]->min_cf2 = new char*[cc_graph[i]->member.size()];
		// initialize empty string for each member
		for(int ii=0;ii<cc_graph[i]->member.size();ii++){
			cc_graph[i]->min_cf1[ii] = new char[nr_free];
			cc_graph[i]->min_cf2[ii] = new char[nr_free];
		}
		order_ctr = 0;
		// loop until every element in the CC has been ordered
		while(order_ctr < cc_graph[i]->member.size()){
			// find nodes which have no parents and mark them
			for(int ii=0;ii<cc_graph[i]->member.size();ii++){

				if(cc_graph[i]->member[ii]->flag == F_REMOVED) continue;

				// if a parent1 exists
				if(cc_graph[i]->member[ii]->pid[0] != 0){
					// and the parent1 has not been removed
					if(cc_graph[i]->member[ii]->pp[0]->flag != F_REMOVED) continue;
				}

				// if a parent2 exists
				if(cc_graph[i]->member[ii]->pid[1] != 0){
					// and the parent2 has not been removed
					if(cc_graph[i]->member[ii]->pp[1]->flag != F_REMOVED) continue;
				}

				// if it has passed all checks, flag it for removal
				cc_graph[i]->member[ii]->flag = F_MARKED;
			}
			// remove all flagged nodes
			for(int ii=0;ii<cc_graph[i]->member.size();ii++){
				if(cc_graph[i]->member[ii]->flag == F_MARKED){
					cc_graph[i]->member[ii]->flag = F_REMOVED;
					cc_swap(i,order_ctr,ii);
					order_ctr++;
				}
			}
		}
	}


	// convert each connected component to an array (the array will be used from now on)
	
	bool removed = false;
	
	for(int i=0;i<cc_graph.size();i++){
		cc_graph[i]->size = cc_graph[i]->member.size();
		cc_graph[i]->node = new Gen*[cc_graph[i]->size];
		for(int ii=0;ii<cc_graph[i]->size;ii++){
			cc_graph[i]->node[ii] = cc_graph[i]->member[ii];
			if(!fill_template(cc_graph[i]->node[ii])){
				fprintf(stderr,"mendelian inconsistency at No.%d\n",cc_graph[i]->node[ii]->id);
				if(!MODE_drop) exit(0);
				fprintf(stderr,"dropping pedigree\n");
				// remove this pedigree
				nr_gen -= cc_graph[i]->size;
				cc_graph.erase(cc_graph.begin() + i);
				i--;
				removed = true;
				num_dropped++;
				break;
			}
		}
		if(removed) removed = false;
		else cc_graph[i]->member.clear();
	}
}
